package com.ruoyi.generator.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.generator.domain.Community.*;
import com.ruoyi.generator.mapper.community.*;
import com.ruoyi.generator.service.ICommunityArticleService;
import com.ruoyi.generator.vo.CommunityArticleVo;
import com.ruoyi.generator.vo.CommunityCircleVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 社区文章管理
 *
 * @author fangzhen
 */
@Api(tags = "社区文章管理")
@RestController
@RequestMapping("/community/article")
public class CommunityArticleController extends BaseController {
    @Autowired
    private ICommunityArticleService communityArticleService;

    @Autowired
    private CommunityClassMapper communityClassMapper;

    @Autowired
    private CommunityArticleCommentMapper communityArticleCommentMapper;

    @Autowired
    private CommunityCommentLikeMapper communityCommentLikeMapper;

    @Autowired
    private CommunityUserCircleMapper communityUserCircleMapper;
    @Autowired
    private CommunityCircleMapper communityCircleMapper;


    /**
     * 获取文章列表信息
     */
    @ApiOperation("获取文章列表")
    @GetMapping("/list")
    //@Anonymous
    public TableDataInfo communityList(CommunityArticle communityArticle) {
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        List<CommunityArticleVo> list = communityArticleService.selectCommunityArticleList(communityArticle, pageNum, pageSize);
        return getDataTable(list);
    }

    /**
     * 发布文章
     */
    @ApiOperation("发布文章")
    @PostMapping("/article")
    //@Anonymous
    public AjaxResult article(@RequestBody CommunityArticle communityArticle) {
        communityArticleService.insertCommunityArticle(communityArticle);
        return AjaxResult.success("文章发布成功!");
    }


    @GetMapping("/class")
    @ApiOperation("获取文章分类列表")
    //@Anonymous
    public AjaxResult getClassList() {
        List<CommunityClass> communityClasses = communityClassMapper.selectList(new QueryWrapper<CommunityClass>());
        return AjaxResult.success(communityClasses);
    }

    @GetMapping("/circleList")
    @ApiOperation("获取文章分类列表")
    //@Anonymous
    public AjaxResult getCircleList() {
        List<CommunityCircle> communityClasses = communityCircleMapper.selectList(new QueryWrapper<CommunityCircle>());
        return AjaxResult.success(communityClasses);
    }


    @ApiOperation("文章评价")
    @PostMapping("/comment")
    public AjaxResult comment(@RequestBody CommunityArticleComment communityArticleComment) throws ParseException {
        if (null == communityArticleComment.getArticleId()) {
            return AjaxResult.error("文章不存在或异常!");
        }

        if (null == communityArticleComment.getUserId()) {
            return AjaxResult.error("用户不存在或异常!");
        }

        //新增评论
        communityArticleService.sendComment(communityArticleComment);

        return AjaxResult.success();
    }

    @ApiOperation("文章评价")
    @DeleteMapping("/comment")
    public AjaxResult deleteComment(@RequestBody CommunityArticleComment communityArticleComment) {
//        if (null == communityArticleComment.getArticleId()) {
//            return AjaxResult.error("文章不存在或异常!");
//        }

        if (null == communityArticleComment.getUserId()) {
            return AjaxResult.error("用户不存在或异常!");
        }

        if (!SecurityUtils.getUserId().equals(communityArticleComment.getUserId())) {
            return AjaxResult.error("不允许删除别人的评论!");
        }

        //删除评论
        communityArticleService.deleteComment(communityArticleComment);

        return AjaxResult.success();
    }

    /**
     * 文章点赞
     */
    @ApiOperation("文章点赞")
    @PostMapping("/like")
    //@Anonymous
    public AjaxResult like(@RequestBody CommunityLike communityLike) {
        if (Objects.isNull(communityLike.getArticleId()) || Objects.isNull(communityLike.getUserId())) {
            return AjaxResult.error("文章或用户信息异常!");
        }
        boolean isSuccess = true;
        String message = "";
        CommunityLike like = communityArticleService.selectCommunityLikeById(communityLike);
        if (Objects.isNull(like)) {
            //新增点赞
            communityLike.setCreateBy(String.valueOf(SecurityUtils.getLoginUser().getUserId()));
            communityLike.setUpdateBy(String.valueOf(SecurityUtils.getLoginUser().getUserId()));
            communityLike.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
            communityLike.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
            isSuccess = communityArticleService.insertCommunityLikeById(communityLike);
            message = "感谢点赞";
        } else {
            //删除点赞
            isSuccess = communityArticleService.deleteCommunityLikeById(like);
            message = "取消点赞";
        }

        if (!isSuccess) {
            message = "操作失败!";
            return AjaxResult.error(message);
        }
        return AjaxResult.success(message);
    }


    /**
     * 获取文章列表信息
     */
    @ApiOperation("文章收藏")
    @PostMapping("/collect")
    //@Anonymous
    public AjaxResult collect(@RequestBody CommunityArticleCollect communityArticleCollect) {
        if (Objects.isNull(communityArticleCollect.getArticleId()) || Objects.isNull(communityArticleCollect.getUserId())) {
            return AjaxResult.error("文章或用户信息异常!");
        }
        boolean isSuccess = true;
        String message = "";
        CommunityArticleCollect collect = communityArticleService.selectCommunityCollectById(communityArticleCollect);
        if (Objects.isNull(collect) || Objects.isNull(collect.getId())) {
            //新增收藏
            isSuccess = communityArticleService.insertCommunityCollectById(communityArticleCollect);
            message = "收藏成功";
        } else {
            //删除收藏
            isSuccess = communityArticleService.deleteCommunityCollectById(collect);
            message = "取消收藏";
        }

        if (!isSuccess) {
            message = "操作失败!";
            return AjaxResult.error(message);
        }
        return AjaxResult.success(message);
    }


    /**
     * 评论点赞
     */
    @ApiOperation("评论点赞")
    @PostMapping("/commentLike")
    //@Anonymous
    public AjaxResult commentLike(@RequestBody CommunityCommentLike commentLike) {
        if (Objects.isNull(commentLike.getCommentId())) {
            return AjaxResult.error("评论或用户信息异常!");
        }
        Long userId = SecurityUtils.getLoginUser().getUserId();
        boolean isSuccess = true;
        String message = "";
        //查询该评论是否已点赞
        CommunityCommentLike communityCommentLike = communityCommentLikeMapper
                .selectOne(new QueryWrapper<CommunityCommentLike>()
                        .eq("comment_id", commentLike.getCommentId())
                        .eq("user_id", userId));

        if (Objects.isNull(communityCommentLike) || Objects.isNull(communityCommentLike.getId())) {
            //新增点赞
            commentLike.setCreateBy(userId);
            commentLike.setUserId(userId);
            commentLike.setUpdateBy(userId);
            commentLike.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
            commentLike.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
            communityCommentLikeMapper.insert(commentLike);
            message = "感谢点赞";
        } else {
            //删除点赞
            communityCommentLikeMapper.deleteById(communityCommentLike);
            message = "取消点赞";
        }

        if (!isSuccess) {
            message = "操作失败!";
            return AjaxResult.error(message);
        }
        return AjaxResult.success(message);
    }


    @ApiOperation("获取圈子信息")
    @GetMapping("/circle")
    //@Anonymous
    public AjaxResult circle(@RequestParam("classId") String classId) {
        if (Strings.isBlank(classId)) {
            return AjaxResult.error("板块信息有误!");
        }
        List<CommunityCircleVo> communityCircles = communityArticleService.selectCommunityCircleList(classId);
        return AjaxResult.success(communityCircles);
    }

    @ApiOperation("我的圈子")
    @GetMapping("/myCircle")
    //@Anonymous
    public AjaxResult myCircle() {
        Long userId = SecurityUtils.getLoginUser().getUserId();
        List<CommunityUserCircle> communityUserCircles = communityUserCircleMapper.selectList(new QueryWrapper<CommunityUserCircle>()
                .eq("user_id", userId)
                .eq("is_delete", 0));

        List<Long> circleIds = new ArrayList<>();
        for (CommunityUserCircle communityUserCircle : communityUserCircles) {
            circleIds.add(communityUserCircle.getCircleId());
        }
        List<CommunityCircle> communityCircles = communityCircleMapper.selectBatchIds(circleIds);
        return AjaxResult.success(communityCircles);
    }


    @ApiOperation("收藏/取关圈子")
    @PostMapping("/circle")
    //@Anonymous
    public AjaxResult circle(@RequestBody CommunityUserCircle communityUserCircle) {
        if (Objects.isNull(communityUserCircle.getCircleId())) {
            return AjaxResult.error("关注圈子异常,请刷新页面重试!");
        }
        Long userId = SecurityUtils.getLoginUser().getUserId();
        CommunityUserCircle userCircle = communityUserCircleMapper
                .selectOne(new QueryWrapper<CommunityUserCircle>()
                        .eq("circle_id", communityUserCircle.getCircleId())
                        .eq("user_id", userId)
                        .eq("is_delete", 0));
        String msg = "";
        if (Objects.isNull(userCircle) || Objects.isNull(userCircle.getId())) {
            //如果未找到数据,则进行关注
            userCircle = new CommunityUserCircle();
            userCircle.setCircleId(communityUserCircle.getCircleId());
            userCircle.setUserId(userId);
            userCircle.setCreateBy(userId);
            userCircle.setUpdateBy(userId);
            userCircle.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
            userCircle.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
            communityUserCircleMapper.insert(userCircle);
            msg = "已关注";
        } else {
            //更新删除标识
            userCircle.setDelete(true);
            userCircle.setCreateBy(userId);
            userCircle.setUpdateBy(userId);
            userCircle.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
            userCircle.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
            communityUserCircleMapper.updateById(userCircle);
            msg = "已取关";
        }
        return AjaxResult.success(msg);
    }

    /**
     * 关注用户
     *
     * @param likeUserId 被关注人id
     * @return 当前关注信息
     */

    @ApiOperation("关注/取关用户")
    @PostMapping("/likeUser")
    public AjaxResult userLike(@RequestParam("likeUserId") Long likeUserId) {
        if (Objects.isNull(likeUserId)) {
            return AjaxResult.error("参数失效!");
        }
        String msg = "";
        CommunityUserLike communityUserLike = communityArticleService.addOrDeleteUserLike(likeUserId);
        if (Objects.isNull(communityUserLike) || Objects.isNull(communityUserLike.getId())) {
            msg = "取消关注";
        } else {
            msg = "已关注";
        }
        return AjaxResult.success(msg);
    }

    @ApiOperation("获取用户信息")
    @GetMapping("/userInfo")
    public AjaxResult userInfo(Long userId) {
        if (Objects.isNull(userId)) {
            userId = SecurityUtils.getLoginUser().getUserId();
        }
        return AjaxResult.success(communityArticleService.selectCommunityUserInfoById(userId));
    }

    @ApiOperation("获取关注列表")
    @GetMapping("/likeUserList")
    public AjaxResult likeUserList() {
        return AjaxResult.success(communityArticleService.selectCommunityUserLikeList(SecurityUtils.getLoginUser().getUserId()));
    }

    @ApiOperation("获取粉丝列表")
    @GetMapping("/userFans")
    public AjaxResult userFans() {
        return AjaxResult.success(communityArticleService.selectUserFansList(SecurityUtils.getLoginUser().getUserId()));
    }

    @ApiOperation("发送用户验证码")
    @GetMapping("/sendSms")
    //todo
    public AjaxResult sendSms() {
        return AjaxResult.success();
    }

}